@@ -25,8 +25,9 @@
 
 
 import gtk
-import gobject
+import pango
 
+from gpodder import util
 from gpodder.liblogger import log
 
 import atexit
@@ -38,6 +39,7 @@ import ConfigParser
 gPodderSettings = {
     # General settings
     'player': ( str, 'xdg-open' ),
+    'videoplayer': (str, 'unspecified'),
     'opml_url': ( str, 'http://gpodder.berlios.de/directory.opml' ),
     'http_proxy': ( str, '' ),
     'ftp_proxy': ( str, '' ),
@@ -52,7 +54,7 @@ gPodderSettings = {
 
     # Boolean config flags
     'update_on_startup': ( bool, False ),
-    'download_after_update': ( bool, False ),
+    'auto_download_when_minimized': (bool, False),
     'use_gnome_bittorrent': ( bool, True ),
     'only_sync_not_played': ( bool, False ),
     'proxy_use_environment': ( bool, True ),
@@ -61,6 +63,27 @@ gPodderSettings = {
     'on_sync_mark_played': ( bool, False ),
     'on_sync_delete': ( bool, False ),
     'auto_remove_old_episodes': ( bool, False ),
+    'auto_update_feeds': (bool, False),
+    'auto_update_frequency': (int, 20),
+    'episode_list_descriptions': (bool, True),
+    'show_toolbar': (bool, True),
+    'ipod_write_gtkpod_extended': (bool, False),
+    
+    # Tray icon and notification settings
+    'display_tray_icon': (bool, False),
+    'minimize_to_tray': (bool, False),  
+    'start_iconified': (bool, False),
+    'enable_notifications': (bool, True),
+    'on_quit_ask': (bool, True),
+
+    # Bluetooth-related settings
+    'bluetooth_enabled': (bool, False),
+    'bluetooth_ask_always': (bool, True),
+    'bluetooth_ask_never': (bool, False),
+    'bluetooth_device_name': (str, 'No device'),
+    'bluetooth_device_address': (str, '00:00:00:00:00:00'),
+    'bluetooth_use_converter': (bool, False),
+    'bluetooth_converter': (str, ''),
 
     # Settings that are updated directly in code
     'ipod_mount': ( str, '/media/ipod' ),
@@ -71,6 +94,8 @@ gPodderSettings = {
     # Special settings (not in preferences)
     'default_new': ( int, 1 ),
     'use_si_units': ( bool, False ),
+    'on_quit_systray': (bool, False),
+    'create_m3u_playlists': (bool, False),
 
     # Window and paned positions
     'main_window_x': ( int, 100 ),
@@ -89,6 +114,8 @@ class Config(dict):
         self.__filename = filename
         self.__section = 'gpodder-conf-1'
         self.__ignore_window_events = False
+        # Name, Type, Value, Type(python type), Editable?, Font weight
+        self.__model = gtk.ListStore(str, str, str, object, bool, int)
 
         atexit.register( self.__atexit)
 
@@ -130,13 +157,23 @@ class Config(dict):
             togglebutton.connect( 'toggled', lambda togglebutton: setattr( self, name, togglebutton.get_active()))
         else:
             raise ValueError( '%s is not a setting' % name)
+    
+    def filechooser_selection_changed(self, name, filechooser):
+        filename = filechooser.get_filename()
+        if filename is not None:
+            setattr(self, name, filename)
 
-    def connect_gtk_filechooser( self, name, filechooser):
+    def connect_gtk_filechooser(self, name, filechooser, is_for_files=False):
         if name in self.Settings:
-            filechooser.set_current_folder(getattr( self, name))
-            filechooser.connect( 'selection-changed', lambda filechooser: setattr( self, name, filechooser.get_filename()))
+            if is_for_files:
+                # A FileChooser for a single file
+                filechooser.set_filename(getattr(self, name))
+            else:
+                # A FileChooser for a folder
+                filechooser.set_current_folder(getattr(self, name))
+            filechooser.connect('selection-changed', lambda filechooser: self.filechooser_selection_changed(name, filechooser))
         else:
-            raise ValueError( '%s is not a setting' % name)
+            raise ValueError('%s is not a setting'%name)
 
     def receive_configure_event( self, widget, event, config_prefix):
         ( x, y, width, height ) = map( lambda x: config_prefix + '_' + x, [ 'x', 'y', 'width', 'height' ])
@@ -160,13 +197,13 @@ class Config(dict):
             window.resize( getattr( self, width), getattr( self, height))
             window.move( getattr( self, x), getattr( self, y))
             self.disable_window_events()
-            gobject.idle_add(self.enable_window_events)
+            util.idle_add(self.enable_window_events)
             window.connect( 'configure-event', self.receive_configure_event, config_prefix)
         else:
             raise ValueError( 'Missing settings in set: %s' % ', '.join( ( x, y, width, height )))
 
     def schedule_save( self):
-        if self.__save_thread == None:
+        if self.__save_thread is None:
             self.__save_thread = threading.Thread( target = self.save_thread_proc)
             self.__save_thread.start()
 
@@ -204,13 +241,16 @@ class Config(dict):
         if filename != None:
             self.__filename = filename
 
+        self.__model.clear()
+
         parser = ConfigParser.RawConfigParser()
         try:
             parser.read( self.__filename)
         except:
             pass
 
-        for ( key, ( fieldtype, default ) ) in self.Settings.items():
+        for key in sorted(self.Settings):
+            (fieldtype, default) = self.Settings[key]
             try:
                 if fieldtype == int:
                     value = parser.getint( self.__section, key)
@@ -224,6 +264,48 @@ class Config(dict):
                 value = default
 
             self[key] = value
+            if value == default:
+                weight = pango.WEIGHT_NORMAL
+            else:
+                weight = pango.WEIGHT_BOLD
+            self.__model.append([key, self.type_as_string(fieldtype), str(value), fieldtype, fieldtype is not bool, weight])
+
+    def model(self):
+        return self.__model
+
+    def toggle_flag(self, name):
+        if name in self.Settings:
+            (fieldtype, default) = self.Settings[name]
+            if fieldtype == bool:
+                setattr(self, name, not getattr(self, name))
+            else:
+                log('Cannot toggle value: %s (not boolean)', name, sender=self)
+        else:
+            log('Invalid setting name: %s', name, sender=self)
+
+    def update_field(self, name, new_value):
+        if name in self.Settings:
+            (fieldtype, default) = self.Settings[name]
+            try:
+                new_value = fieldtype(new_value)
+            except:
+                log('Cannot convert "%s" to %s. Ignoring.', str(new_value), fieldtype.__name__, sender=self)
+                return False
+            setattr(self, name, new_value)
+            return True
+        else:
+            log('Invalid setting name: %s', name, sender=self)
+            return False
+
+    def type_as_string(self, type):
+        if type == int:
+            return _('Integer')
+        elif type == float:
+            return _('Float')
+        elif type == bool:
+            return _('Boolean')
+        else:
+            return _('String')
 
     def __setattr__( self, name, value):
         if name in self.Settings:
@@ -232,6 +314,14 @@ class Config(dict):
                 if self[name] != fieldtype(value):
                     log( 'Update: %s = %s', name, value, sender = self)
                     self[name] = fieldtype(value)
+                    for row in self.__model:
+                        if row[0] == name:
+                            row[2] = str(fieldtype(value))
+                            if self[name] == default:
+                                weight = pango.WEIGHT_NORMAL
+                            else:
+                                weight = pango.WEIGHT_BOLD
+                            row[5] = weight
                     self.schedule_save()
             except:
                 raise ValueError( '%s has to be of type %s' % ( name, fieldtype.__name__ ))
