@@ -25,7 +25,6 @@
 
 import gtk
 import gtk.gdk
-import gobject
 import thread
 import threading
 import urllib
@@ -46,23 +45,16 @@ from liblogger import log
 
 import shlex
 
-# my gpodderlib variable
-g_podder_lib = None
-
-# some awkward kind of "singleton" ;)
-def gPodderLib():
-    global g_podder_lib
-    if g_podder_lib == None:
-        g_podder_lib = gPodderLibClass()
-    return g_podder_lib
-
-class gPodderLibClass( object):
+class gPodderLib(object):
     def __init__( self):
+        log('Creating gPodderLib()', sender=self)
         gpodder_dir = os.path.expanduser( '~/.config/gpodder/')
         util.make_directory( gpodder_dir)
 
-        self.feed_cache_file = os.path.join( gpodder_dir, 'feedcache.db')
-        self.channel_settings_file = os.path.join( gpodder_dir, 'channelsettings.db')
+        self.tempdir = gpodder_dir
+        self.feed_cache_file = os.path.join(gpodder_dir, 'feedcache.pickle.db')
+        self.channel_settings_file = os.path.join(gpodder_dir, 'channelsettings.pickle.db')
+        self.episode_metainfo_file = os.path.join(gpodder_dir, 'episodemetainfo.pickle.db')
 
         self.channel_opml_file = os.path.join(gpodder_dir, 'channels.opml')
         self.channel_xml_file = os.path.join(gpodder_dir, 'channels.xml')
@@ -74,6 +66,13 @@ class gPodderLibClass( object):
         self.config = config.Config( os.path.join( gpodder_dir, 'gpodder.conf'))
         util.make_directory(self.config.bittorrent_dir)
 
+        # We need to make a seamless upgrade, so by default the video player is not specified
+        # so the first time this application is run it will detect this and set it to the same 
+        # as the audio player.  This keeps gPodder functionality identical to that prior to the 
+        # upgrade.   The user can then set a specific video player if they so wish.	
+        if self.config.videoplayer == 'unspecified':
+            self.config.videoplayer = self.config.player	
+
         self.__download_history = HistoryStore( os.path.join( gpodder_dir, 'download-history.txt'))
         self.__playback_history = HistoryStore( os.path.join( gpodder_dir, 'playback-history.txt'))
         self.__locked_history = HistoryStore( os.path.join( gpodder_dir, 'lock-history.txt'))
@@ -122,8 +121,8 @@ class gPodderLibClass( object):
             log( 'Warning: Called get_device_name() when no device was selected.', sender = self)
             return '(unknown device)'
 
-    def format_filesize( self, bytesize):
-        return util.format_filesize( bytesize, self.config.use_si_units)
+    def format_filesize(self, bytesize, digits=2):
+        return util.format_filesize(bytesize, self.config.use_si_units, digits)
 
     def clean_up_downloads( self, delete_partial = False):
         # Clean up temporary files left behind by old gPodder versions
@@ -197,7 +196,18 @@ class gPodderLibClass( object):
         self.history_mark_played( episode.url)
         filename = episode.local_filename()
 
-        command_line = shlex.split( util.format_desktop_command( self.config.player, filename).encode('utf-8'))
+        # Determine the file type and set the player accordingly.  
+        file_type = util.file_type_by_extension(util.file_extension_from_url(episode.url))
+
+        if file_type == 'video':
+            player = self.config.videoplayer
+        elif file_type == 'audio':
+            player = self.config.player
+        else:
+            log('Non-audio or video file type, using xdg-open for %s', filename, sender=self)
+            player = 'xdg-open'
+ 
+        command_line = shlex.split(util.format_desktop_command(player, filename).encode('utf-8'))
         log( 'Command line: [ %s ]', ', '.join( [ '"%s"' % p for p in command_line ]), sender = self)
         try:
             subprocess.Popen( command_line)
@@ -205,20 +215,12 @@ class gPodderLibClass( object):
             return ( False, command_line[0] )
         return ( True, command_line[0] )
 
-    def open_folder( self, folder):
-        try:
-            subprocess.Popen( [ 'xdg-open', folder ])
-            # FIXME: Win32-specific "open" code needed here
-            # as fallback when xdg-open not available
-        except:
-            log( 'Cannot open folder: "%s"', folder, sender = self)
-
     def image_download_thread( self, url, callback_pixbuf = None, callback_status = None, callback_finished = None, cover_file = None):
         if callback_status != None:
-            gobject.idle_add( callback_status, _('Downloading channel cover...'))
+            util.idle_add(callback_status, _('Downloading channel cover...'))
         pixbuf = gtk.gdk.PixbufLoader()
         
-        if cover_file == None:
+        if cover_file is None:
             log( 'Downloading %s', url)
             pixbuf.write( urllib.urlopen(url).read())
         
@@ -230,7 +232,12 @@ class gPodderLibClass( object):
         
         if cover_file != None:
             log( 'Reading cover from %s', cover_file)
-            pixbuf.write( open( cover_file, "r").read())
+            try:
+                pixbuf.write( open( cover_file, "r").read())
+            except:
+                # Probably a data error, delete temp file
+                log('Data error while reading pixbuf. Deleting %s', cover_file, sender=self)
+                util.delete_file(cover_file)
         
         try:
             pixbuf.close()
@@ -248,11 +255,11 @@ class gPodderLibClass( object):
                 if pb.get_height() > MAX_SIZE:
                     factor = MAX_SIZE*1.0/pb.get_height()
                     pb = pb.scale_simple( int(pb.get_width()*factor), int(pb.get_height()*factor), gtk.gdk.INTERP_BILINEAR)
-                gobject.idle_add( callback_pixbuf, pb)
+                util.idle_add(callback_pixbuf, pb)
         if callback_status != None:
-            gobject.idle_add( callback_status, '')
+            util.idle_add(callback_status, '')
         if callback_finished != None:
-            gobject.idle_add( callback_finished)
+            util.idle_add(callback_finished)
 
     def get_image_from_url( self, url, callback_pixbuf = None, callback_status = None, callback_finished = None, cover_file = None):
         if not url and not os.path.exists( cover_file):
@@ -266,7 +273,7 @@ class gPodderLibClass( object):
         self.history_mark_played( url)
 
         if self.config.use_gnome_bittorrent:
-            if util.find_command( 'gnome-btdownload') == None:
+            if util.find_command('gnome-btdownload') is None:
                 log( 'Cannot find "gnome-btdownload". Please install gnome-bittorrent.', sender = self)
                 return False
 
@@ -341,3 +348,6 @@ class HistoryStore( types.ListType):
         return affected
 
 
+# Global, singleton gPodderLib object
+gl = gPodderLib()
+
