@@ -38,8 +38,8 @@ from gpodder import cache
 from gpodder import services
 from gpodder import draw
 
-from liblogger import log
-import libgpodder
+from gpodder.liblogger import log
+from gpodder.libgpodder import gl
 
 from os.path import exists
 from os.path import basename
@@ -73,12 +73,12 @@ import md5
 
 import string
 
-import shelve
+from gpodder import dumbshelve
 
 global_lock = threading.RLock()
 
 class ChannelSettings(object):
-    storage = shelve.open( libgpodder.gPodderLib().channel_settings_file)
+    storage = dumbshelve.open_shelve(gl.channel_settings_file)
 
     @classmethod
     def get_settings_by_url( cls, url):
@@ -98,12 +98,29 @@ class ChannelSettings(object):
         cls.storage.sync()
 
 
+class EpisodeURLMetainfo(object):
+    storage = dumbshelve.open_shelve(gl.episode_metainfo_file)
+
+    @classmethod
+    def get_metadata_by_url(cls, url):
+        if isinstance(url, unicode):
+            url = url.encode('utf-8')
+        if cls.storage.has_key(url):
+            return cls.storage[url]
+        else:
+            log('Trying to download metainfo for %s', url)
+            result = util.get_episode_info_from_url(url, gl.config.http_proxy)
+            cls.storage[url] = result
+            cls.storage.sync()
+            return result
+
+
 class podcastChannel(ListType):
     """holds data for a complete channel"""
     SETTINGS = ('sync_to_devices', 'is_music_channel', 'device_playlist_name','override_title','username','password')
     icon_cache = {}
 
-    storage = shelve.open( libgpodder.gPodderLib().feed_cache_file)
+    storage = dumbshelve.open_shelve(gl.feed_cache_file)
     fc = cache.Cache( storage)
 
     @classmethod
@@ -189,7 +206,7 @@ class podcastChannel(ListType):
         self.username = ''
         self.password = ''
 
-        self.update_save_dir_size()
+        self.save_dir_size = 0
 
         self.__tree_model = None
 
@@ -252,8 +269,6 @@ class podcastChannel(ListType):
         ChannelSettings.set_settings_by_url( self.url, settings)
 
     def newest_pubdate_downloaded( self):
-        gl = libgpodder.gPodderLib()
-
         # Try DownloadHistory's entries first
         for episode in self:
             if gl.history_is_downloaded( episode.url):
@@ -266,7 +281,6 @@ class podcastChannel(ListType):
         return pubdate
     
     def episode_is_new(self, episode, last_pubdate = None):
-        gl = libgpodder.gPodderLib()
         if last_pubdate is None:
             last_pubdate = self.newest_pubdate_downloaded()
 
@@ -286,10 +300,9 @@ class podcastChannel(ListType):
     
     def get_new_episodes( self):
         last_pubdate = self.newest_pubdate_downloaded()
-        gl = libgpodder.gPodderLib()
 
         if not last_pubdate:
-            return self[0:min(len(self),gl.config.default_new)]
+            return [episode for episode in self[0:min(len(self),gl.config.default_new)] if self.episode_is_new(episode)]
 
         new_episodes = []
         for episode in self.get_all_episodes():
@@ -308,32 +321,52 @@ class podcastChannel(ListType):
 
         return True
     
+    def update_m3u_playlist(self, downloaded_episodes=None):
+        if gl.config.create_m3u_playlists:
+            if downloaded_episodes is None:
+                downloaded_episodes = self.load_downloaded_episodes()
+            fn = util.sanitize_filename(self.title)
+            if len(fn) == 0:
+                fn = os.path.basename(self.save_dir)
+            m3u_filename = os.path.join(gl.downloaddir, fn+'.m3u')
+            log('Writing playlist to %s', m3u_filename, sender=self)
+            f = open(m3u_filename, 'w')
+            f.write('#EXTM3U\n')
+            for episode in sorted(downloaded_episodes):
+                filename = episode.local_filename()
+                if os.path.dirname(filename).startswith(os.path.dirname(m3u_filename)):
+                    filename = filename[len(os.path.dirname(m3u_filename)+os.sep):]
+                f.write('#EXTINF:0,'+self.title+' - '+episode.title+' ('+episode.pubDate+')\n')
+                f.write(filename+'\n')
+            f.close()
+    
     def addDownloadedItem( self, item):
         # no multithreaded access
         global_lock.acquire()
 
         downloaded_episodes = self.load_downloaded_episodes()
         already_in_list = item.url in [ episode.url for episode in downloaded_episodes ]
-
+        
         # only append if not already in list
         if not already_in_list:
             downloaded_episodes.append( item)
             self.save_downloaded_episodes( downloaded_episodes)
 
             # Update metadata on file (if possible and wanted)
-            if libgpodder.gPodderLib().config.update_tags and tagging_supported():
+            if gl.config.update_tags and tagging_supported():
                 filename = item.local_filename()
                 try:
                     update_metadata_on_file( filename, title = item.title, artist = self.title)
                 except:
                     log('Error while calling update_metadata_on_file() :(')
 
-        libgpodder.gPodderLib().history_mark_downloaded( item.url)
+        gl.history_mark_downloaded(item.url)
+        self.update_m3u_playlist(downloaded_episodes)
         
         if item.file_type() == 'torrent':
             torrent_filename = item.local_filename()
             destination_filename = util.torrent_filename( torrent_filename)
-            libgpodder.gPodderLib().invoke_torrent( item.url, torrent_filename, destination_filename)
+            gl.invoke_torrent(item.url, torrent_filename, destination_filename)
             
         global_lock.release()
         return not already_in_list
@@ -404,26 +437,31 @@ class podcastChannel(ListType):
     def iter_set_downloading_columns( self, model, iter, new_episodes = []):
         url = model.get_value( iter, 0)
         local_filename = model.get_value( iter, 8)
-        played = not libgpodder.gPodderLib().history_is_played( url)
-        locked = libgpodder.gPodderLib().history_is_locked(url)
+        played = not gl.history_is_played(url)
+        locked = gl.history_is_locked(url)
+
+        if gl.config.episode_list_descriptions:
+            icon_size = 32
+        else:
+            icon_size = 16
 
         if os.path.exists( local_filename):
-            file_type = util.file_type_by_extension( util.file_extension_from_url( url))
+            file_type = util.file_type_by_extension( util.file_extension_from_url(url))
             if file_type == 'audio':
-                status_icon = util.get_tree_icon('audio-x-generic', played, locked, self.icon_cache)
+                status_icon = util.get_tree_icon('audio-x-generic', played, locked, self.icon_cache, icon_size)
             elif file_type == 'video':
-                status_icon = util.get_tree_icon('video-x-generic', played, locked, self.icon_cache)
+                status_icon = util.get_tree_icon('video-x-generic', played, locked, self.icon_cache, icon_size)
             elif file_type == 'torrent':
-                status_icon = util.get_tree_icon('applications-internet', played, locked, self.icon_cache)
+                status_icon = util.get_tree_icon('applications-internet', played, locked, self.icon_cache, icon_size)
             else:
-                status_icon = util.get_tree_icon('unknown', played, locked, self.icon_cache)
+                status_icon = util.get_tree_icon('unknown', played, locked, self.icon_cache, icon_size)
             
-        elif services.download_status_manager.is_download_in_progress( url):
-            status_icon = util.get_tree_icon( gtk.STOCK_GO_DOWN, icon_cache = self.icon_cache)
-        elif libgpodder.gPodderLib().history_is_downloaded( url):
-            status_icon = util.get_tree_icon( gtk.STOCK_DELETE, icon_cache = self.icon_cache)
-        elif url in [ e.url for e in new_episodes ]:
-            status_icon = util.get_tree_icon( gtk.STOCK_NEW, icon_cache = self.icon_cache)
+        elif services.download_status_manager.is_download_in_progress(url):
+            status_icon = util.get_tree_icon(gtk.STOCK_GO_DOWN, icon_cache=self.icon_cache, icon_size=icon_size)
+        elif gl.history_is_downloaded(url):
+            status_icon = util.get_tree_icon(gtk.STOCK_DELETE, icon_cache=self.icon_cache, icon_size=icon_size)
+        elif url in [e.url for e in new_episodes]:
+            status_icon = util.get_tree_icon(gtk.STOCK_NEW, icon_cache=self.icon_cache, icon_size=icon_size)
         else:
             status_icon = None
 
@@ -437,7 +475,11 @@ class podcastChannel(ListType):
         new_episodes = self.get_new_episodes()
 
         for item in self.get_all_episodes():
-            new_iter = new_model.append( ( item.url, item.title, libgpodder.gPodderLib().format_filesize( item.length), True, None, item.cute_pubdate(), item.one_line_description(), item.description, item.local_filename() ))
+            if gl.config.episode_list_descriptions:
+                description = '%s\n<small>%s</small>' % (saxutils.escape(item.title), saxutils.escape(item.one_line_description()))
+            else:
+                description = saxutils.escape(item.title)
+            new_iter = new_model.append((item.url, item.title, gl.format_filesize(item.length, 1), True, None, item.cute_pubdate(), description, item.description, item.local_filename()))
             self.iter_set_downloading_columns( new_model, new_iter, new_episodes)
         
         self.update_save_dir_size()
@@ -451,7 +493,7 @@ class podcastChannel(ListType):
         return None
 
     def get_save_dir(self):
-        save_dir = os.path.join( libgpodder.gPodderLib().downloaddir, self.filename, '')
+        save_dir = os.path.join(gl.downloaddir, self.filename, '')
 
         # Create save_dir if it does not yet exist
         if not util.make_directory( save_dir):
@@ -497,6 +539,7 @@ class podcastChannel(ListType):
                     downloaded_episodes.remove( episode)
 
         self.save_downloaded_episodes( downloaded_episodes)
+        self.update_m3u_playlist(downloaded_episodes)
         global_lock.release()
 
 class podcastItem(object):
@@ -529,8 +572,23 @@ class podcastItem(object):
         if not episode.url:
             raise ValueError( 'Episode has an invalid URL')
 
+        if not episode.pubDate:
+            metainfo = episode.get_metainfo()
+            if 'pubdate' in metainfo:
+                episode.pubDate = metainfo['pubdate']
+
         if hasattr( enclosure, 'length'):
-            episode.length = enclosure.length
+            try:
+                episode.length = int(enclosure.length)
+            except:
+                episode.length = -1
+
+        # For episodes with a small length amount, try to find it via HTTP HEAD
+        if episode.length <= 100:
+            metainfo = episode.get_metainfo()
+            if 'length' in metainfo:
+                episode.length = metainfo['length']
+
         if hasattr( enclosure, 'type'):
             episode.mimetype = enclosure.type
 
@@ -552,29 +610,20 @@ class podcastItem(object):
         self.channel = channel
         self.pubDate = ''
 
+    def get_metainfo(self):
+        return EpisodeURLMetainfo.get_metadata_by_url(self.url)
+
     def is_played(self):
-        gl = libgpodder.gPodderLib()
         return gl.history_is_played(self.url)
 
     def age_in_days(self):
-        dt = util.file_modification_datetime(self.local_filename())
-        if dt is None:
-            return 0
-        else:
-            return (datetime.now()-dt).days
+        return util.file_age_in_days(self.local_filename())
 
     def is_old(self):
-        gl = libgpodder.gPodderLib()
         return self.age_in_days() > gl.config.episode_old_age
     
     def get_age_string(self):
-        age = self.age_in_days()
-        if age == 1:
-            return _('one day ago')
-        elif age > 1:
-            return _('%d days ago') % age
-        else:
-            return ''
+        return util.file_age_to_string(self.age_in_days())
 
     age_prop = property(fget=get_age_string)
 
@@ -583,17 +632,13 @@ class podcastItem(object):
         if not lines or lines[0] == '':
             return _('No description available')
         else:
-            desc = lines[0].strip()
-            if len( desc) > 84:
-                return desc[:80] + '...'
-            else:
-                return desc
+            return ' '.join((l.strip() for l in lines if l.strip() != ''))
 
     def is_downloaded( self):
         return os.path.exists( self.local_filename())
 
     def is_locked(self):
-        return libgpodder.gPodderLib().history_is_locked(self.url)
+        return gl.history_is_locked(self.url)
 
     def delete_from_disk(self):
         try:
@@ -606,8 +651,8 @@ class podcastItem(object):
         return os.path.join( self.channel.save_dir, md5.new( self.url).hexdigest() + extension)
 
     def sync_filename( self):
-        if libgpodder.gPodderLib().config.custom_sync_name_enabled:
-            return util.object_string_formatter( libgpodder.gPodderLib().config.custom_sync_name, episode = self, channel = self.channel)
+        if gl.config.custom_sync_name_enabled:
+            return util.object_string_formatter(gl.config.custom_sync_name, episode=self, channel=self.channel)
         else:
             return self.title
 
@@ -627,6 +672,10 @@ class podcastItem(object):
             return '00000000'
     
     def __cmp__( self, other):
+        if self.pubDate == other.pubDate:
+            log('pubDate equal, comparing titles (buggy feed?)', sender=self)
+            return cmp(self.title, other.title)
+
         try:
             timestamp_self = int(mktime_tz( parsedate_tz( self.pubDate)))
             timestamp_other = int(mktime_tz( parsedate_tz( other.pubDate)))
@@ -678,12 +727,11 @@ class podcastItem(object):
 
     def calculate_filesize( self):
         try:
-            self.length = str(os.path.getsize( self.local_filename()))
+            self.length = os.path.getsize(self.local_filename())
         except:
             log( 'Could not get filesize for %s.', self.url)
 
     def get_filesize_string( self):
-        gl = libgpodder.gPodderLib()
         return gl.format_filesize( self.length)
 
     filesize_prop = property(fget=get_filesize_string)
@@ -702,7 +750,7 @@ class podcastItem(object):
     played_prop = property(fget=get_played_string)
     
     def equals( self, other_item):
-        if other_item == None:
+        if other_item is None:
             return False
         
         return self.url == other_item.url
@@ -758,7 +806,7 @@ def channels_to_model(channels):
 
 
 def load_channels( load_items = True, force_update = False, callback_proc = None, callback_url = None, callback_error = None, offline = False):
-    importer = opml.Importer( libgpodder.gPodderLib().channel_opml_file)
+    importer = opml.Importer(gl.channel_opml_file)
     result = []
 
     urls_to_keep = []
@@ -775,7 +823,7 @@ def load_channels( load_items = True, force_update = False, callback_proc = None
     return result
 
 def save_channels( channels):
-    exporter = opml.Exporter( libgpodder.gPodderLib().channel_opml_file)
+    exporter = opml.Exporter(gl.channel_opml_file)
     return exporter.write(channels)
 
 
