@@ -29,10 +29,11 @@ are not tied to any specific part of gPodder.
 
 """
 
-
+import gpodder
 from gpodder.liblogger import log
 
 import gtk
+import gobject
 
 import os
 import os.path
@@ -40,15 +41,21 @@ import glob
 import stat
 
 import re
-import htmlentitydefs
+import subprocess
+from htmlentitydefs import entitydefs
 import time
 import locale
 import gzip
 import datetime
+import threading
 
 import urlparse
 import urllib
 import urllib2
+import httplib
+import webbrowser
+
+import feedparser
 
 import StringIO
 import xml.dom.minidom
@@ -140,6 +147,9 @@ def calculate_size( path):
     correct if the user doesn't have appropriate permissions 
     to list all subdirectories of the given path.
     """
+    if path is None:
+        return 0L
+
     if os.path.dirname( path) == '/':
         return 0L
 
@@ -166,6 +176,9 @@ def file_modification_datetime(filename):
     as a datetime.datetime object or None if the modification
     date cannot be determined.
     """
+    if filename is None:
+        return None
+
     if not os.access(filename, os.R_OK):
         return None
 
@@ -178,6 +191,38 @@ def file_modification_datetime(filename):
         return None
 
 
+def file_age_in_days(filename):
+    """
+    Returns the age of the specified filename in days or
+    zero if the modification date cannot be determined.
+    """
+    dt = file_modification_datetime(filename)
+    if dt is None:
+        return 0
+    else:
+        return (datetime.datetime.now()-dt).days
+
+
+def file_age_to_string(days):
+    """
+    Converts a "number of days" value to a string that
+    can be used in the UI to display the file age.
+
+    >>> file_age_to_string(0)
+    ''
+    >>> file_age_to_string(1)
+    'one day ago'
+    >>> file_age_to_String(2)
+    '2 days ago'
+    """
+    if days == 1:
+        return _('one day ago')
+    elif days > 1:
+        return _('%d days ago') % days
+    else:
+        return ''
+
+
 def get_free_disk_space(path):
     """
     Calculates the free disk space available to the current user
@@ -196,7 +241,7 @@ def get_free_disk_space(path):
     return s.f_bavail * s.f_bsize
 
 
-def format_filesize( bytesize, use_si_units = False):
+def format_filesize(bytesize, use_si_units=False, digits=2):
     """
     Formats the given size in bytes to be human-readable, 
 
@@ -235,7 +280,7 @@ def format_filesize( bytesize, use_si_units = False):
             used_value = bytesize / float(value)
             used_unit = unit
 
-    return '%.2f %s' % ( used_value, used_unit )
+    return ('%.'+str(digits)+'f %s') % (used_value, used_unit)
 
 
 def delete_file( path):
@@ -255,24 +300,28 @@ def delete_file( path):
         pass
 
 
-def remove_html_tags( html):
+
+def remove_html_tags(html):
     """
     Remove HTML tags from a string and replace numeric and
     named entities with the corresponding character, so the 
     HTML text can be displayed in a simple text view.
     """
-    # strips html from a string (fix for <description> tags containing html)
-    rexp = re.compile( "<[^>]*>")
-    stripstr = rexp.sub( '', html)
-    # replaces numeric entities with entity names
-    dict = htmlentitydefs.codepoint2name
-    for key in dict.keys():
-        stripstr = stripstr.replace( '&#'+str(key)+';', '&'+unicode( dict[key], 'iso-8859-1')+';')
-    # strips html entities
-    dict = htmlentitydefs.entitydefs
-    for key in dict.keys():
-        stripstr = stripstr.replace( '&'+unicode(key,'iso-8859-1')+';', unicode(dict[key], 'iso-8859-1'))
-    return stripstr
+    # If we would want more speed, we could make these global
+    re_strip_tags = re.compile('<[^>]*>')
+    re_unicode_entities = re.compile('&#(\d{2,4});')
+    re_html_entities = re.compile('&(.{2,8});')
+
+    # Remove all HTML/XML tags from the string
+    result = re_strip_tags.sub('', html)
+
+    # Convert numeric XML entities to their unicode character
+    result = re_unicode_entities.sub(lambda x: unichr(int(x.group(1))), result)
+
+    # Convert named HTML entities to their unicode character
+    result = re_html_entities.sub(lambda x: unicode(entitydefs.get(x.group(1),''), 'iso-8859-1'), result)
+
+    return result
 
 
 def torrent_filename( filename):
@@ -344,7 +393,7 @@ def file_type_by_extension( extension):
     """
     types = {
             'audio': [ 'mp3', 'ogg', 'wav', 'wma', 'aac', 'm4a' ],
-            'video': [ 'mp4', 'avi', 'mpg', 'mpeg', 'm4v', 'mov', 'divx' ],
+            'video': [ 'mp4', 'avi', 'mpg', 'mpeg', 'm4v', 'mov', 'divx', 'flv', 'wmv', '3gp' ],
             'torrent': [ 'torrent' ],
     }
 
@@ -363,7 +412,7 @@ def file_type_by_extension( extension):
     return None
 
 
-def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=None):
+def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=None, icon_size=32):
     """
     Loads an icon from the current icon theme at the specified
     size, suitable for display in a gtk.TreeView.
@@ -379,23 +428,23 @@ def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=Non
     the cache.
     """
 
-    if icon_cache != None and (icon_name,add_bullet,add_padlock) in icon_cache:
-        return icon_cache[(icon_name,add_bullet,add_padlock)]
+    if icon_cache != None and (icon_name,add_bullet,add_padlock,icon_size) in icon_cache:
+        return icon_cache[(icon_name,add_bullet,add_padlock,icon_size)]
     
     icon_theme = gtk.icon_theme_get_default()
 
     try:
-        icon = icon_theme.load_icon( icon_name, 16, 0)
+        icon = icon_theme.load_icon(icon_name, icon_size, 0)
     except:
         log( '(get_tree_icon) Warning: Cannot load icon with name "%s", will use  default icon.', icon_name)
-        icon = icon_theme.load_icon( gtk.STOCK_DIALOG_QUESTION, 16, 0)
+        icon = icon_theme.load_icon(gtk.STOCK_DIALOG_QUESTION, icon_size, 0)
 
     if icon and (add_bullet or add_padlock):
         # We'll modify the icon, so use .copy()
         if add_bullet:
             try:
                 icon = icon.copy()
-                emblem = icon_theme.load_icon(gtk.STOCK_YES, 10, 0)
+                emblem = icon_theme.load_icon(gtk.STOCK_YES, int(float(icon_size)*1.2/3.0), 0)
                 size = emblem.get_width()
                 pos = icon.get_width() - size
                 emblem.composite(icon, pos, pos, size, size, pos, pos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
@@ -404,14 +453,14 @@ def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=Non
         if add_padlock:
             try:
                 icon = icon.copy()
-                emblem = icon_theme.load_icon('emblem-nowrite', 8, 0)
+                emblem = icon_theme.load_icon('emblem-nowrite', int(float(icon_size)/2.0), 0)
                 size = emblem.get_width()
                 emblem.composite(icon, 0, 0, size, size, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
             except:
                 log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
 
     if icon_cache != None:
-        icon_cache[(icon_name,add_bullet,add_padlock)] = icon
+        icon_cache[(icon_name,add_bullet,add_padlock,icon_size)] = icon
 
     return icon
 
@@ -585,3 +634,234 @@ def itunes_discover_rss(url):
         return None
 
 
+def idle_add(func, *args):
+    """
+    This is a wrapper function that does the Right
+    Thing depending on if we are running a GTK+ GUI or
+    not. If not, we're simply calling the function.
+
+    If we are a GUI app, we use gobject.idle_add() to
+    call the function later - this is needed for
+    threads to be able to modify GTK+ widget data.
+    """
+    if gpodder.interface_is_gui:
+        def x(f, *a):
+            f(*a)
+            return False
+
+        gobject.idle_add(func, *args)
+    else:
+        func(*args)
+
+
+def discover_bluetooth_devices():
+    """
+    This is a generator function that returns
+    (address, name) tuples of all nearby bluetooth
+    devices found.
+
+    If the user has python-bluez installed, it will
+    be used. If not, we're trying to use "hcitool".
+
+    If neither python-bluez or hcitool are available,
+    this function is the empty generator.
+    """
+    try:
+        # If the user has python-bluez installed
+        import bluetooth
+        log('Using python-bluez to find nearby bluetooth devices')
+        for name, addr in bluetooth.discover_devices(lookup_names=True):
+            yield (name, addr)
+    except:
+        if find_command('hcitool') is not None:
+            log('Using hcitool to find nearby bluetooth devices')
+            # If the user has "hcitool" installed
+            p = subprocess.Popen(['hcitool', 'scan'], stdout=subprocess.PIPE)
+            for line in p.stdout:
+                match = re.match('^\t([^\t]+)\t([^\t]+)\n$', line)
+                if match is not None:
+                    (addr, name) = match.groups()
+                    yield (name, addr)
+        else:
+            log('Cannot find either python-bluez or hcitool - no bluetooth?')
+            return # <= empty generator
+
+
+def bluetooth_send_file(filename, device=None, callback_finished=None):
+    """
+    Sends a file via bluetooth using gnome-obex send.
+    Optional parameter device is the bluetooth address
+    of the device; optional parameter callback_finished
+    is a callback function that will be called when the
+    sending process has finished - it gets one parameter
+    that is either True (when sending succeeded) or False
+    when there was some error.
+
+    This function tries to use "bluetooth-sendto", and if
+    it is not available, it also tries "gnome-obex-send".
+    """
+    command_line = None
+
+    if find_command('bluetooth-sendto'):
+        command_line = ['bluetooth-sendto']
+        if device is not None:
+            command_line.append('--device=%s' % device)
+    elif find_command('gnome-obex-send'):
+        command_line = ['gnome-obex-send']
+        if device is not None:
+            command_line += ['--dest', device]
+
+    if command_line is not None:
+        command_line.append(filename)
+        result = (subprocess.Popen(command_line).wait() == 0)
+        if callback_finished is not None:
+            callback_finished(result)
+        return result
+    else:
+        log('Cannot send file. Please install "bluetooth-sendto" or "gnome-obex-send".')
+        if callback_finished is not None:
+            callback_finished(False)
+        return False
+        
+        
+def format_seconds_to_hour_min_sec(seconds):
+    """
+    Take the number of seconds and format it into a
+    human-readable string (duration).
+
+    >>> format_seconds_to_hour_min_sec(3834)
+    '1 hour, 3 minutes and 54 seconds'
+    >>> format_seconds_to_hour_min_sec(2600)
+    '1 hour'
+    >>> format_seconds_to_hour_min_sec(62)
+    '1 minute and 2 seconds'
+    """
+
+    if seconds < 1:
+        return _('0 seconds')
+
+    result = []
+
+    hours = seconds/3600
+    seconds = seconds%3600
+
+    minutes = seconds/60
+    seconds = seconds%60
+
+    if hours == 1:
+        result.append(_('1 hour'))
+    elif hours > 1:
+        result.append(_('%i hours') % hours)
+
+    if minutes == 1:
+        result.append(_('1 minute'))
+    elif minutes > 1:
+        result.append(_('%i minutes') % minutes)
+
+    if seconds == 1:
+        result.append(_('1 second'))
+    elif seconds > 1:
+        result.append(_('%i seconds') % seconds)
+
+    if len(result) > 1:
+        return (' '+_('and')+' ').join((', '.join(result[:-1]), result[-1]))
+    else:
+        return result[0]
+
+
+def get_episode_info_from_url(url, proxy=None):
+    """
+    Try to get information about a podcast episode by sending
+    a HEAD request to the HTTP server and parsing the result.
+
+    The return value is a dict containing all fields that 
+    could be parsed from the URL. This currently contains:
+    
+      "length": The size of the file in bytes
+      "pubdate": A formatted representation of the pubDate
+
+    If the "proxy" parameter is used, it has to be the URL 
+    of the HTTP proxy server to use, e.g. http://proxy:8080/
+    
+    If there is an error, this function returns {}. This will
+    only function with http:// and https:// URLs.
+    """
+    if not (url.startswith('http://') or url.startswith('https://')):
+        return {}
+
+    if proxy is None or proxy.strip() == '':
+        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(url)
+        conn = httplib.HTTPConnection(netloc)
+        start = len(scheme) + len('://') + len(netloc)
+        conn.request('HEAD', url[start:])
+    else:
+        (scheme, netloc, path, parms, qry, fragid) = urlparse.urlparse(proxy)
+        conn = httplib.HTTPConnection(netloc)
+        conn.request('HEAD', url)
+
+    r = conn.getresponse()
+    result = {}
+
+    log('Trying to get metainfo for %s', url)
+
+    if 'content-length' in r.msg:
+        try:
+            length = int(r.msg['content-length'])
+            result['length'] = length
+        except ValueError, e:
+            log('Error converting content-length header.')
+
+    if 'last-modified' in r.msg:
+        try:
+            parsed_date = feedparser._parse_date(r.msg['last-modified'])
+            pubdate = updated_parsed_to_rfc2822(parsed_date)
+            result['pubdate'] = pubdate
+        except:
+            log('Error converting last-modified header.')
+
+    return result
+
+
+def gui_open(filename):
+    """
+    Open a file or folder with the default application set
+    by the Desktop environment. This uses "xdg-open".
+    """
+    try:
+        subprocess.Popen(['xdg-open', filename])
+        # FIXME: Win32-specific "open" code needed here
+        # as fallback when xdg-open not available
+    except:
+        log('Cannot open file/folder: "%s"', folder, sender=self, traceback=True)
+
+
+def open_website(url):
+    """
+    Opens the specified URL using the default system web
+    browser. This uses Python's "webbrowser" module, so
+    make sure your system is set up correctly.
+    """
+    threading.Thread(target=webbrowser.open, args=(url,)).start()
+
+
+def sanitize_filename(filename):
+    """
+    Generate a sanitized version of a filename that can
+    be written on disk (i.e. remove/replace invalid 
+    characters and encode in the native language)
+    """
+    # Try to detect OS encoding (by Leonid Ponomarev)
+    if 'LANG' in os.environ and '.' in os.environ['LANG']:
+        lang = os.environ['LANG']
+        (language, encoding) = lang.rsplit('.', 1)
+        log('Detected encoding: %s', encoding)
+        enc = encoding
+    else:
+        # Using iso-8859-15 here as (hopefully) sane default
+        # see http://en.wikipedia.org/wiki/ISO/IEC_8859-1
+        log('Using ISO-8859-15 as encoding. If this')
+        log('is incorrect, please set your $LANG variable.')
+        enc = 'iso-8859-15'
+
+    return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(enc, 'ignore'))
+
